#!/usr/bin/ruby

require 'term/ansicolor'
include Term::ANSIColor

# Task: prohibit optimization on fewer and fewer .o files until
# uncovering the .o file that breaks a given test.
# 
# We prohibit optimization by removing an optimized .o file and
# building with optimizations disabled.

# The test we plan to run; it should succeed without
# optimizations and fail with optimizations enabled.
# Else this script is useless.
# Test = "../../secret-412/1-fork/test-forkbomb-nosmp.py"
Test = "../../secret-412/1-fork/test-forkbomb.py"

# all geekos kernel object files, some of which
# we'll disable optimiztions for.
All_object_files = Dir.glob('geekos/*.o')

def bail(msg)
  print msg
  exit 1
end

def friendlify_object_files(partition)
  partition.map { |p| p.gsub(/^geekos\/(\w+).o/, '\\1') }.sort.join(', ')
end
 

def check_when_disabling_set(partition)
  # first, clean it all.
  Kernel.system("make clean > /dev/null")
  bail "clean failed" if $? != 0 

  # small opt 
  if !(All_object_files - partition).empty? 
    Kernel.system("make -j4 OPTIMIZE=-O2 > /dev/null")
    if $? != 0 
      bail "build failed"
    end
    puts "#{yellow}#{bold}disabling optimization for the following .o files:#{reset} %s" % friendlify_object_files(partition)
    File.delete(*partition) # delete takes arguments; splat them
  end
  
  Kernel.system("make -j4 OPTIMIZE=-O0")
  if $? != 0 
    bail "partial build failed"
  end

  current = Dir.pwd
  Dir.chdir(File.dirname(Test))
  puts "#{yellow}#{bold}running #{File.basename(Test)} disabling optimization for:#{reset} %s" % friendlify_object_files(partition)
  Kernel.system("./#{File.basename(Test)}")
  Dir.chdir(current)

  puts $?
  if $? == 0 then
    puts "#{green}#{bold}disabling optimization on #{friendlify_object_files(partition)} allowed it to work#{reset}"
    true
  else
    puts "#{red}#{bold}disabling optimization on #{friendlify_object_files(partition)} did not fix#{reset}"
    false
  end
end


partition = All_object_files.shuffle[0..All_object_files.length / 2]

worked_overall = check_when_disabling_set(All_object_files)
if !worked_overall then
  puts "disabling optimization did not allow #{File.basename(Test)} to pass; no sense searching."
  exit 1
end
worked_partition = check_when_disabling_set(partition)
worked_twiddle_partition = check_when_disabling_set(All_object_files - partition)

puts "#{worked_partition} #{worked_twiddle_partition}"
if worked_partition && !worked_twiddle_partition 
  puts "the culprit is in #{partition}"
elsif !worked_partition && worked_twiddle_partition 
  puts "the culprit is in #{All_object_files - partition}"
elsif worked_partition && worked_twiddle_partition 
  puts "#{bold}hooray, #{File.basename(Test)} worked with both optimizations#{reset}"
  worked_optimized = check_when_disabling_set([])
  if worked_optimized then
    puts "#{bold}hooray, #{File.basename(Test)} worked with both optimizations and all optimized.#{reset}"
    exit 0
  else
    puts "#{red}#{bold}boo, #{File.basename(Test)} worked with both partitions, but not with all optimized.#{reset}"
    exit 1
  end
end


